/* -*-C-*- */

/* $Id: DBIF.xs 1.44 2000/03/07
   Copyright (c) 1998-2000 Boris Jakubith

*/

#include <stdio.h>
#include "dbif.h"
#include "prefs.h"
#include "tools.h"

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

MODULE = DBIF		PACKAGE = DBIF		PREFIX = dbif_

U32
constant()
    PROTOTYPE:
    ALIAS:
    RT_USER		= 1
    RT_USER		= 2
    CC_USER		= 3
    RT_LINK		= 4
    CC_LINK		= 5
    RT_NSUSER		= 6
    CC_NSUSER		= 7
    RT_ADMIN		= 8
    CC_ADMIN		= 9
    RT_DOMAIN		= 10
    CC_DOMAIN		= 11
    RT_DOMLINK		= 12
    CC_DOMLINK		= 13
    RT_NSDOMAIN		= 14
    CC_NSDOMAIN		= 15
    RT_LLINK		= 16
    CC_LLINK		= 17
    RT_MAILSYS		= 18
    CC_MAILSYS		= 19
    RT_IP4TRANS		= 20
    CC_IP4TRANS		= 21
    RT_IP6TRANS		= 22
    CC_IP6TRANS		= 23
    RT_REDIRECT		= 24
    CC_REDIRECT		= 25
    RT_IDENTITY		= 26
    CC_IDENTITY		= 27
    RT_VXTS		= 28
    CC_VXTS		= 29
    RT_INVALID		= 30
    CC_INVALID		= 31
    RT_FILTER		= 32
    CC_FILTER		= 33
    RT_BSMTP		= 34
    CC_BSMTP		= 35
    CC_DOMLIKE		= 36
    CC_IPTRANS		= 37
    CC_SPECIAL		= 38
    CC_RUSERS		= 39
    CC_DENTRIES		= 40
    CC_TRUEDOM		= 41
    DBIF_NOERR		= 42
    DBIF_ENOMEM		= 43
    DBIF_ACFAIL		= 44
    DBIF_WRFAIL		= 45
    DBIF_RDFAIL		= 46
    DBIF_SLFAIL		= 47
    DBIF_NOWR		= 48
    DBIF_NORD		= 49
    DBIF_NOIT		= 50
    DBIF_NODB		= 51
    DBIF_INT		= 52
    DBIF_DATA		= 53
    DBIF_CCFAIL		= 54
    CODE:
    if (!ix) {
	char *mt = GvNAME(CvGV(cv));
	croak ("Unknown DBIF entry type constant `%s'", mt);
    } else {
	static I32 values[] = {
	    RT_USER, RT_USER, CC_USER, RT_LINK, CC_LINK, RT_NSUSER, CC_NSUSER,
	    RT_ADMIN, CC_ADMIN, RT_DOMAIN, CC_DOMAIN, RT_DOMLINK, CC_DOMLINK,
	    RT_NSDOMAIN, CC_NSDOMAIN, RT_LLINK, CC_LLINK, RT_MAILSYS,
	    CC_MAILSYS, RT_IP4TRANS, CC_IP4TRANS, RT_IP6TRANS, CC_IP6TRANS,
	    RT_REDIRECT, CC_REDIRECT, RT_IDENTITY, CC_IDENTITY, RT_VXTS,
	    CC_VXTS, RT_INVALID, CC_INVALID, RT_FILTER, CC_FILTER, RT_BSMTP,
	    CC_BSMTP, CC_DOMLIKE, CC_IPTRANS, CC_SPECIAL, CC_RUSERS,
	    CC_DENTRIES, CC_TRUEDOM, DBIF_NOERR, DBIF_ENOMEM, DBIF_ACFAIL,
	    DBIF_WRFAIL, DBIF_RDFAIL, DBIF_SLFAIL, DBIF_NOWR, DBIF_NORD,
	    DBIF_NOIT, DBIF_NODB, DBIF_INT, DBIF_DATA, DBIF_CCFAIL };
	RETVAL = values[ix - 1];
    }
    OUTPUT:
    RETVAL

DBIF_FILE
dbif_reset (dbname)
    char *dbname
    PREINIT:
	DBIF_FILE dbf;
    CODE:
	dbif_reset (&dbf, dbname);
	RETVAL = dbf;
    OUTPUT:
	RETVAL

DBIF_FILE
dbif_create (dbname, mode)
    char *dbname
    int mode
    PREINIT:
	DBIF_FILE dbf;
    CODE:
	dbif_create (&dbf, dbname, mode);
	RETVAL = dbf;
    OUTPUT:
	RETVAL

DBIF_FILE
dbif_rw (dbname)
    char *dbname
    PREINIT:
	DBIF_FILE dbf;
    CODE:
	dbif_rw (&dbf, dbname);
	RETVAL = dbf;
    OUTPUT:
	RETVAL

void
dbif_close (dbf)
    DBIF_FILE dbf
    CODE:
	dbif_close (dbf);

void
dbif_sync (dbf)
    DBIF_FILE dbf
    CODE:
	dbif_sync (dbf);

## dbif_resolve db, em_addr, res_addr, inv
## dbif_resolve db, em_addr, inv
##                      ==> (passwd, folder, rectype)
##                      ==> (passwd, folder, rectype, fld_limit, msg_limit)
##                      ==> (passwd, folder, rectype, fld_limit, msg_limit,
##                           num_users, max_users)
##
void
dbif_resolve (db, em_addr, r_addr = 0, inv)
    DBIF_FILE db
    char *em_addr
    char *r_addr
    int inv
    PREINIT:
	dbif_rec *r;
	char *res_addr;
    PPCODE:
	if (r_addr != 0) {
	    r = dbif_resolve (db, em_addr, &res_addr, inv);
	    sv_setpv ((SV *)r_addr,
		      sv_2mortal(newSVpv(res_addr, strlen(res_addr))));
	    free (res_addr);
	} else {
	    r = dbif_resolve (db, em_addr, NULL, inv);
	}
	if (r != NULL) {
	    int results = 3, i, nu, mu;
	    int rt = dbif_rectype (r);
	    if ((1 << rt) & (CC_TRUEDOM|CC_RUSERS)) { results += 2; }
	    if ((1 << rt) & CC_TRUEDOM) { results += 2; }
	    EXTEND(sp, results);
	    PUSHs(sv_2mortal(newSVpv(r->passwd, strlen(r->passwd))));
	    PUSHs(sv_2mortal(newSVpv(r->folder, strlen(r->folder))));
	    PUSHs(sv_2mortal(newSViv(dbif_rectype(r))));
	    if (results > 3) {
		PUSHs(sv_2mortal(newSViv(r->fld_limit)));
		PUSHs(sv_2mortal(newSViv(r->msg_limit)));
	    }
	    if (results > 5) {
		dbif_getlim (r, &nu, &mu);
		PUSHs(sv_2mortal(newSViv(nu)));
		PUSHs(sv_2mortal(newSViv(mu)));
	    }
	    dbif_free (r);
	}

## dbif_get db, em_addr ==> (passwd, folder, rectype)
##                      ==> (passwd, folder, rectype, fld_limit, msg_limit)
##                      ==> (passwd, folder, rectype, fld_limit, msg_limit,
##                           num_users, max_users)
##
void
dbif_get (db, em_addr)
    DBIF_FILE db
    char *em_addr
    PREINIT:
	dbif_rec *r;
    PPCODE:
	r = dbif_get (db, em_addr);
	if (r != NULL) {
	    int results = 3, i, nu, mu;
	    int rt = dbif_rectype (r);
	    if ((1 << rt) & (CC_TRUEDOM|CC_RUSERS)) { results += 2; }
	    if ((1 << rt) & CC_TRUEDOM) { results += 2; }
	    EXTEND(sp, results);
	    PUSHs(sv_2mortal(newSVpv(r->passwd, strlen(r->passwd))));
	    PUSHs(sv_2mortal(newSVpv(r->folder, strlen(r->folder))));
	    PUSHs(sv_2mortal(newSViv(rt)));
	    if (results > 3) {
		PUSHs(sv_2mortal(newSViv(r->fld_limit)));
		PUSHs(sv_2mortal(newSViv(r->msg_limit)));
	    }
	    if (results > 5) {
		dbif_getlim (r, &nu, &mu);
		PUSHs(sv_2mortal(newSViv(nu)));
		PUSHs(sv_2mortal(newSViv(mu)));
	    }
	    dbif_free (r);
	}


I32
dbif_insert (db, em_addr, pwd, fld, rt, fldlim=0, msglim=0, nu=0, mu=0)
    DBIF_FILE db
    char *em_addr
    char *pwd
    char *fld
    int rt
    off_t fldlim
    size_t msglim
    int nu
    int mu
    ALIAS:
	DBIF::set = 1
    PREINIT:
	dbif_rec *r;
    CODE:
	r = dbif_mkrec (pwd, fld, fldlim, msglim, rt, nu, mu);
	if (ix == 0) {
	    RETVAL = dbif_insert (db, em_addr, r);
	} else {
	    RETVAL = dbif_set (db, em_addr, r);
	}
	dbif_free (r);

I32
dbif_del (db, em_addr)
    DBIF_FILE db
    char *em_addr

void
getconf (conffile = 0)
    char *conffile;
    CODE:
	if (conffile != 0) {
	    prefs.config_name = gdupl (__FILE__, __LINE__, conffile);
	}
	getconf();

I32
dbif_errno ()

DBIF_CURSOR
dbif_select (db, key, relcode)
    DBIF_FILE db
    char *key
    int relcode

void
dbif_first (dbc)
    DBIF_CURSOR dbc
    PREINIT:
	dbif_rec *r;
	char *em_addr;
	int nu, mu;
    ALIAS:
	dbif_next = 1
    PPCODE:
	em_addr = (ix == 1 ? dbif_next (dbc, &r) : dbif_first (dbc, &r));
	if (em_addr != NULL) {
	    int results = 4;
	    int rt = dbif_rectype (r);
	    if ((1 << rt) & (CC_TRUEDOM|CC_RUSERS)) { results += 2; }
	    if ((1 << rt) & CC_TRUEDOM) { results += 2; }
	    EXTEND(sp, results);
	    PUSHs(sv_2mortal(newSVpv(em_addr, strlen (em_addr))));
	    PUSHs(sv_2mortal(newSVpv(r->passwd, strlen(r->passwd))));
	    PUSHs(sv_2mortal(newSVpv(r->folder, strlen(r->folder))));
	    PUSHs(sv_2mortal(newSViv(dbif_rectype(r))));
	    if (results > 4) {
		PUSHs(sv_2mortal(newSViv(r->fld_limit)));
		PUSHs(sv_2mortal(newSViv(r->msg_limit)));
	    }
	    if (results > 5) {
		dbif_getlim (r, &nu, &mu);
		PUSHs(sv_2mortal(newSViv(nu)));
		PUSHs(sv_2mortal(newSViv(mu)));
	    }
	    dbif_free (r);
	    free (em_addr);
	}

I32
dbif_end (dbc)
    DBIF_CURSOR dbc
