/* vxusercheck.c
**
** Domain checker - for use in conjunction with the `smail' configuration
** file `/etc/smail/routers'
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "config.h"
#include "prefs.h"
#include "conf.h"
#include "dbif.h"
#include "tools.h"
#include "vxerror.h"
#include "version.h"
#include "vxopt.h"

void usage (void)
{
    fprintf (stderr, "\nUsage: %s [-f <configfile>] <e-mail address> ..."
		     "\n       %s -h"
		     "\n       %s -V\n"
		     "\n   -f <configfile>"
		     "\n      read configuration from file <configfile>"
		     "\n      (default: %s)"
		     "\n   -h"
		     "\n      show this help"
		     "\n   -V"
		     "\n      show the program's version number\n\n",
		     progname, progname, progname, CONFIGFILE);
    exit (EX_USAGE);
}

#define LINKTYPE (CC_LINK|CC_DOMLINK|CC_LLINK|CC_IP4TRANS|CC_IP6TRANS|CC_REDIRECT)
char *rectypes[] = {
    " regular user", " strong link", " non VxMail-user", " admin user",
    " regular domain", " domain link", " non VxMail-domain", " weak link",
    "n IPv4 translation", "n IPv6 translation", " domain redirection",
    " database identity entry", " vxts secret entry", "n invalid user",
    " mail filter", " bsmtp user"
};

int main (int argc, char *argv[])
{
    char *vxuser;
    DBIF_FILE userdb;
    dbif_rec *urec;
    int ix, rt;

    vxopt (&argc, argv, usage);
    reset_umask();

    if (argc < 2) {
	fprintf (stderr, "%s: missing argument! Please call `%s -h' "
			 "for help!\n",
			 progname,
			 progname);
	exit (EX_USAGE);
    }
    getconf ();
    get_ug ();
    dbif_reset (&userdb, prefs.userdb_name);
    if (!userdb) {
	vxerror_at (EX_SOFTWARE, 0, __FILE__, __LINE__ - 2,
		    "opening user database failed: %s\n",
		    dbif_errormsg ());
    }
    for (ix = 1; ix < argc; ++ix) {
	vxuser = argv[ix];
	setgid (prefs.vxgid);
	setreuid (getuid(), prefs.vxuid);
	urec = dbif_resolve (userdb, vxuser, NULL, 0);
	printf ("USERDB entry `%s' ", vxuser);
	if (!urec) { printf ("doesn't exist!\n"); continue; }
	rt = dbif_rectype (urec);
	printf (" is a%s", rectypes[rt]);
	if ((1 << rt) & (LINKTYPE)) {
	    printf (" to `%s'.", urec->folder);
	}
	printf ("\n");
	dbif_free (urec);
    }
    dbif_close (userdb);
    return 0;
}
