# This is my spec file for RPM based distributions
#
Name: VxMail
Version: 1.50
Release: 3
Copyright: GPL for the package, LGPL for the library `libvxtools.a', examples free
Group: Mail
Source: VxMail-1.50.tgz
Summary: Local delivery agent and POP3 server for implementing virtual mail domains
BuildRoot: /tmp/VxMail
%description
VxMail is a local delivery sub-system and POP3 service which enables holding
more than one e-mail domain on one host. At the moment this package consists of
the programs
    `vxcheck'		- checks folder for limits in age and/or size of
			  messages
    `vxconv' 		- convert smail/sendmail folders into the format
			  required by VxMail
    `vxdeliver'		- deliver mails locally; this program uses the
			  complete recipient address as an index into a user
			  database (GDBM at the moment); this will make this
			  system independant from the system's password
			  database and allows you to hold more than one mail
    `vxdomcheck'	- check if a name supplied as argument corresponds to
			  a domain entry in the user database
    `vxgetconf'		- write the complete configuration of the VxMail system
			  or a single configuration item of it to stdout; you
			  can also find out the compiled configuration defaults
			  by using this tool
    `vxmkdb'		- create the user database from a human readable
			  description
    `vxpop'		- the POP3 server of the VxMail system which can
			  handle smail/sendmail folders, too.
    `vxquery'		- a replacement for `vxdump' and `vxdomcheck'.

%prep
%setup

%build
make SUOPTS="--prefix=/usr --shared-dbif=yes --drac=no --cc=gcc"

%install
strip dist/sbin/*
make DESTDIR="$RPM_BUILD_ROOT" PREFIX=/usr install.de

%files
%doc AUTHORS
%doc Copyright
%doc examples
%config /etc/vxmail.conf
%config /etc/vxmail-redir.txt
%config /etc/vxmail.progs

/usr/sbin/vxcheck
/usr/sbin/vxconv
/usr/sbin/vxdeliver
/usr/sbin/vxdomcheck
/usr/sbin/vxdump
/usr/sbin/vxgetconf
/usr/sbin/vxmkdb
/usr/sbin/vxpop
/usr/sbin/vxquery
/usr/lib/libvxtools.a
/usr/lib/VxMail/*
/usr/include/VxMail/bittypes.h
/usr/include/VxMail/conf.h
/usr/include/VxMail/config.h
/usr/include/VxMail/conv.h
/usr/include/VxMail/dbif.h
/usr/include/VxMail/exitcodes.h
/usr/include/VxMail/exithl.h
/usr/include/VxMail/hlsock.h
/usr/include/VxMail/md5.h
/usr/include/VxMail/prefs.h
/usr/include/VxMail/rc4.h
/usr/include/VxMail/regex.h
/usr/include/VxMail/procttl.h
/usr/include/VxMail/rfc822.h
/usr/include/VxMail/sysauth.h
/usr/include/VxMail/tools.h
/usr/include/VxMail/utypes.h
/usr/include/VxMail/version.h
/usr/include/VxMail/vxerror.h
/usr/include/VxMail/vxopt.h
/usr/include/VxMail/xfile.h
/usr/man/de/man8/vxcheck.8.gz
/usr/man/de/man8/vxconv.8.gz
/usr/man/de/man8/vxdeliver.8.gz
/usr/man/de/man8/vxdomcheck.8.gz
/usr/man/de/man8/vxgetconf.8.gz
/usr/man/de/man8/vxmkdb.8.gz
/usr/man/de/man8/vxpop.8.gz
/usr/man/de/man8/vxquery.8.gz

%pre
#! /bin/sh

if [ -f /etc/vxmail.conf ]; then
    cp /etc/vxmail.conf /etc/vxmail.conf.pre
fi
if [ -f /etc/vxmail.progs ]; then
    cp /etc/vxmail.progs /etc/vxmail.progs.pre
fi
if [ -f /etc/vxmail-redir.txt ]; then
    cp /etc/vxmail-redir.txt /etc/vxmail-redir.txt.pre
fi

cat <<EOT
Your configuration files were saved; after a successful installation/upgrading
i will restore them.
EOT

%post
#! /bin/sh

# At first, change the ownership of the VxMail-files
chown root.root \
    /usr/sbin/vx{check,conv,deliver,domcheck,dump,getconf,mkdb,pop,query}
chmod u+s /usr/sbin/vx{deliver,domcheck,query}
chown root.root /usr/lib/libvxtools.a /usr/lib/VxMail /usr/lib/VxMail/*
chown root.root /usr/include/VxMail /usr/include/VxMail/*
chown root.root \
    /usr/man/de/man8/vx{check,conv,deliver,domcheck,dump,getconf,mkdb,pop,query}.8.gz

# Get the default user/group and working directory
vxuser="`/usr/sbin/vxgetconf vxuser 2>/dev/null`"
vxgroup="`/usr/sbin/vxgetconf vxgroup 2>/dev/null`"
listdir="`/usr/sbin/vxgetconf listdir 2>/dev/null`"

# Create the VxMail list directory
if ! [ -d "$listdir" ]; then
    if [ -e "$listdir" ]; then
	echo "WARNING! I am going to replace the existing file \`$listdir'" 1>&2
	echo "         with the required directory." 1>&2
	rm -f "$listdir"
    fi
    if ! mkdir "$listdir" >/dev/null 2>&1; then
	echo "ERROR! I couldn't create the directory \`$listdir'" 1>&2
	echo "       Please continue the configuration manually!" 1>&2
    fi
else
    echo "WARNING! I am going to change the ownership of the existing" 1>&2
    echo "         \`$listdir' directory." 1>&2
fi

# Create the group $vxgroup if it does not already exist
if ! grep -q "^${vxgroup}:" /etc/group; then
    if [ "$vxgroup" = mail ]; then
        groupadd -g 8 "$vxgroup"
    else
        groupadd "$vxgroup"
    fi
fi

# Create the user $vxuser if it does not already exist
if ! grep -q "^${vxuser}:" /etc/passwd; then
    if [ "$vxuser" = mail ]; then
        useradd -c "VxMail system user" -d /var/spool/mail -g $vxgroup \
                -s /bin/false -u 8 $vxuser
    else
        useradd -c "VxMail system user" -d /var/spool/mail -g $vxgroup \
                -s /bin/false $vxuser
    fi
    vxuser=`grep -- "^${vxuser}:" /etc/passwd | cut -d: -f3`
#    chown "${vxuser}" /var/spool/mail
#    chgrp "${vxgroup}" /var/spool/mail
#    chmod a+rwx,+t /var/spool/mail
fi

chown ${vxuser} "$listdir"
chgrp "${vxgroup}" "$listdir"
chmod 6755 "$listdir"

## At last, we must restore any saved configuration files
if [ -f /etc/vxmail.conf.pre ]; then
    sed -e 's|^service |pop3_service |' </etc/vxmail.conf.pre >/etc/vxmail.conf 
    rm -f /etc/vxmail.conf.pre
fi
if [ -f /etc/vxmail.progs.pre ]; then
    mv /etc/vxmail.progs.pre /etc/vxmail.progs
fi
if [ -f /etc/vxmail-redir.txt.pre ]; then
    mv /etc/vxmail-redir.txt.pre /etc/vxmail-redir.txt
fi
cat <<'EOT'
Your old configuration files were restored and i replaced the no longer existing
option name `service' by the new one `pop3_service'.
EOT

%prerm

saves=''
if [ -f /etc/vxmail.conf ]; then
    saves="$saves /etc/vxmail.conf"
fi
if [ -f /etc/vxmail.progs ]; then
    saves="$saves /etc/vxmail.progs"
fi
if [ -f /etc/vxmail-redir.txt ]; then
    saves="$saves /etc/vxmail-redir.txt"
fi
tar cf /etc/vxmail-conf.tar $saves
cat <<EOT
Your configuration files were saved prior removing the VxMail package; if you
want to remove them too, please do it manually:

    > rm -f /etc/vxmail-conf.tar

.
EOT

%clean
rm -rf "$RPM_BUILD_ROOT"
