/* tools.c
**
** Einige ntzliche Routinen
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdarg.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <signal.h>
#if !(defined(__USE_SVID) || defined(__USE_XOPEN))
/*#warning neither __USE_SVID nor __USE_XOPEN defined!*/
#define timezone __timezone
#endif
#include <time.h>
#include <fcntl.h>

#include "vxerror.h"
#define _TOOLS_C
#include "tools.h"
#include "prefs.h"
#include "mrmacs.h"

static exception_t erraction = (exception_t) 0; 

exception_t setup_exception (exception_t handle)
{
    exception_t tmp = erraction;
    erraction = handle;
    return tmp;
}

typedef struct heap_s *heap_t;
struct heap_s {
    heap_t next;
    size_t sz;
    int free;
};

heap_t mymem = (heap_t) NULL;

void heap_init (void)
{
    mymem = (heap_t) NULL;
}

static int unlookup (void *mem)
{
    heap_t p = (heap_t) ((char *) mem - sizeof (struct heap_s));
    if (p->free) { return 1; }
    p->free = 1;
    p->next = mymem;
    mymem = p;
    return 0;
}

static void *lookup (size_t sz)
{
    heap_t p, q;
    char *res;
    unlessnull (mymem) {
	if (mymem->sz >= sz) {
	    p = mymem; mymem = p->next;
	    p->free = 0;
	    res = (char *) p;
	    res += sizeof (struct heap_s);
	    return (void *) res;
	}
	for (q = mymem, p = q->next; noNULL(p); q = p, p = q->next) {
	    if (p->sz >= sz) {
		q->next = p->next;
		p->free = 0;
		res = (char *) p;
		res += sizeof (struct heap_s);
		return (void *) res;
	    }
	}
    }
    unlessnull (p = (heap_t) malloc (sizeof (struct heap_s) + sz)) {
	p->next = (heap_t) 0;
	p->sz = sz;
	p->free = 0;
	res = (char *) p;
	res += sizeof (struct heap_s);
	return (void *) res;
    }
    return NULL;
}

char *sdupl (const char *string)
{
    char *res = (char *) lookup (strlen (string) + 1);
    unlessnull (res) { strcpy (res, string); }
    return res;
}

char *gdupl (char *file, int line, const char *string)
{
    char *res;
    ifnull (res = sdupl (string)) {
	if (erraction) { erraction ("(gdupl) failed"); }
	vxerror_at (EX_OSERR, errno, file, line, "(gdupl)");
    }
    return res;
}

void *galloc (char *file, int line, size_t size)
{
    void *res;
    ifnull (res = lookup (size)) {
	if (erraction) { erraction ("(gdupl) failed"); }
	vxerror_at (EX_OSERR, errno, file, line, "(galloc)");
    }
    return res;
}

void gfree (void *ref)
{
    if (unlookup (ref)) {
	vxerror (0, 0, "attempt to free an already free'd memory area!\n");
	vxerror (0, 0, "memory area at %08x (%s)\n", ref, ref);
	kill (getpid (), SIGSEGV);
    }
}

static void *grealloc (void *ref, size_t size)
{
    heap_t p = (heap_t) ((char *) ref - sizeof (struct heap_s));
    void *res;
    if (p->sz >= size) { return ref; }
    unlessnull (res = lookup (size)) { memcpy (res, ref, p->sz); gfree (ref); }
    return res;
}

void *gresize (char *file, int line, void *ref, size_t size)
{
    void *res = grealloc (ref, size);
    ifnull (res) {
	vxerror_at (EX_OSERR, errno, file, line, "(gresize)");
    }
    return res;
}

char *vconcat (char **ap)
{
    size_t llen = 0;
    char *res, *p = NULL;
    int ix;
    for (ix = 0; ap[ix]; ++ix) { llen += strlen (ap[ix]); }
    unlessnull (res = lookup (llen + 1)) {
	p = res;
	for (ix = 0; ap[ix]; ++ix) {
	    llen = strlen (ap[ix]);
	    memcpy (p, ap[ix], llen);
	    p += llen;
	    *p = '\0';
	}
	*p = '\0';
    }
    return res;
}

/* concat a NULL-terminated list of string to a new string which is the
** result of this function.
*/
char *concat (char *_1st, ...)
{
    va_list ap, bp;
    char *res, **svec, **p;
    size_t svlen = sizeof (char *);
    ifnull (_1st) {
	ifnull (svec = lookup (svlen)) { return NULL; }
	*svec = NULL;
    } else {
	svlen += sizeof (char *);
	va_start(ap, _1st); va_start (bp, _1st);
	for (res = va_arg(ap, char *); noNULL(res); res = va_arg(ap, char *)) {
	    svlen += sizeof (char *);
	}
	va_end(ap);
	ifnull (svec = lookup (svlen)) { return NULL; }
	p = svec;
	*p++ = _1st;
	for (res = va_arg(bp, char *); noNULL(res); res = va_arg(bp, char *)) {
	    *p++ = res;
	}
	va_end(bp);
	*p = NULL;
    }
    res = vconcat (svec);
    gfree (svec);
    return res;
}

/* concat a NULL-terminated list of string to a new string which is the
** result of this function; abort the program execution if memory for the
** function result couldn't be successfully allocated.
*/
char *gconc (char *file, int line, ...)
{
    va_list ap, bp;
    char *res, **svec, **p;
    size_t svlen = sizeof (char *);
    svlen += sizeof (char *);
    va_start (ap, line);
    va_start (bp, line);
    for (res = va_arg(ap, char *); noNULL(res); res = va_arg(ap, char *)) {
	svlen += sizeof (char *);
    }
    va_end (ap);
    ifnull (svec = lookup (svlen)) {
	vxerror_at (EX_OSERR, errno, file, line, "(gconc:v)");
    }
    p = svec;
    for (res = va_arg(bp, char *); noNULL(res); res = va_arg(bp, char *)) {
	*p++ = res;
    }
    va_end (bp);
    *p = NULL;
    ifnull (res = vconcat (svec)) {
	vxerror_at (EX_OSERR, errno, file, line, "(gconv:s)");
    }
    gfree (svec);
    return res;
}

void gappend (char *file, int line, char **res, ...)
{
    va_list ap, bp;
    char *rv, *p, *q;
    size_t rlen = (isNULL(*res) ? 0 : strlen (*res));
    va_start(ap, res);
    va_start(bp, res);
    for (p = va_arg(ap, char *); noNULL(p); p = va_arg(ap, char *)) {
	rlen += strlen (p);
    }
    ifnull (rv = lookup (rlen + 1)) {
	vxerror_at (EX_OSERR, errno, file, line, "(gappend)");
    }
    q = rv;
    unlessnull (p = *res) {
	rlen = strlen (p); memcpy (q, p, rlen); q += rlen;
    }
    for (p = va_arg(bp, char *); noNULL(p); p = va_arg(bp, char *)) {
	rlen = strlen (p); memcpy (q, p, rlen); q += rlen;
    }
    unlessnull (*res) { gfree (*res); }
    *res = rv;
}

static int noeol = 0;

static time_t _logstamp = (time_t) 0;

static const char *logstamp (void)
{
    static char timestr[60];
    char *p;
/*    if (_logstamp == (time_t) 0) {*/
	(void) time (&_logstamp);
	strftime (timestr, 60, "%m/%d/%Y %H:%M:%S", localtime (&_logstamp));
	p = timestr + strlen (timestr);
	if (*--p == '\r') {
	    *p = '\0';
	} else if (*p == '\n') {
	    *p = '\0'; if (*--p == '\r') { *p = '\0'; }
	}
/*    }*/
    return timestr;
}

void vvxlog (int ll, char *fmt, va_list av)
{
    /* Do no logging if no logfile is open! */
    ifnull (logfile) return;
    /* Otherwise, the logging of the message depends on the current log-level */
    if (loglevel >= ll) {
	if (noeol) {
	    noeol = 0;
	} else {
	    fprintf (logfile, "%s(%d): ", logstamp(), getpid());
/*##was:	    fprintf (logfile, "%5d(%s): ", getpid(), logstamp ());*/
	}
	if (fmt[strlen (fmt) - 1] != '\n') { noeol = 1; }
	vfprintf (logfile, fmt, av);
	fflush (logfile);
    }
}

void vxlog (int ll, char *fmt, ...)
{
    va_list ap;
    va_start (ap, fmt);
    vvxlog (ll, fmt, ap);
    va_end (ap);
}

void npfxlog (char *fmt, ...)
{
    va_list ap;
    va_start (ap, fmt);
    vvxlog (0, fmt, ap);
    va_end (ap);
}

void vxsetlog (char *logname)
{
    ifnull (logname) return;
    if (!strcmp (logname, "-")) {
	logfile = stderr;
    } else ifnull (logfile = fopen (logname, "a")) {
	vxerror_at (EX_OSERR, errno, __FILE__, __LINE__,
		    "opening logfile `%s' failed", logname);
    }
}

void vxerr2log (void)
{
    ifnull (logfile) { return; }
    if (logfile == stderr) { return; }
    if (dup2 (fileno(logfile), 2) == -1) {
	vxerror_at (EX_OSERR, errno, __FILE__, __LINE__ - 1,
		    "redirecting `stderr' to logfile failed");
    }
}

void vxerr2null (void)
{
    int fd = open ("/dev/null", O_WRONLY|O_APPEND, 0666);
    if (fd == -1) {
	vxerror_at (EX_OSERR, errno, __FILE__, __LINE__ - 2,
		    "redirecting `stderr' to `/dev/null' failed");
    }
    if (dup2 (fd, 2) == -1) {
	vxerror_at (EX_OSERR, errno, __FILE__, __LINE__ - 1,
		    "redirecting `stderr' to `/dev/null' failed");
    }
    close (fd);
}

int lccmp (const char *s1, const char *s2, ssize_t n)
{
    if (n < 0) {
	while (*s1 && tolower (*s1) == tolower (*s2)) { ++s1; ++s2; }
	return (int) tolower (*s1) - (int) tolower (*s2);
    }
    while (n-- > 0) {
	if (tolower (*s1++) != tolower (*s2++)) {
	    return (int) tolower(*--s1) - (int) tolower(*--s2);
	}
    }
    return 0;
}

void lcconv(char *bf, int bfl)
{
    if (bfl < 0) {
	while (*bf) { *bf = tolower(*bf); ++bf; }
    } else {
	while (*bf && bfl-- > 0) { *bf = tolower(*bf); ++bf; }
    }
}

char *gvarins (char *file, int line, char *in, char *varname, const char *value)
{
    size_t len, vlen, rlen;
    char *p, *q, *r, *res;
    char *deref = gconc (file, line, "$(", varname, ")", NULL);
    vlen = strlen (value); rlen = strlen (deref);
    len = 0; q = in;
    while (noNULL(p = strstr (q, deref))) {
	len += p - q + vlen;
	q = p + rlen;
    }
    len += strlen (q);
    res = galloc (file, line, len + 1);
    r = res; len = 0; q = in;
    while (noNULL(p = strstr (q, deref))) {
	memcpy (r, q, p - q); r += p - q;
	memcpy (r, value, vlen); r += vlen;
	q = p + rlen;
    }
    strcpy (r, q);
    gfree (deref);
    return res;
}

/* execute a command in a sub-process and redirect it's output to the
** logfile (if one was opened).
*/

int spawn (char *prog, ...)
{
    pid_t child;
    char *arg;
    va_list ap;
    char **av = (char **) galloc (__FILE__, __LINE__, 2 * sizeof(char *));
    int ac = 1, lfh, ofh = 0, efh = 0, status;
    av[0] = prog;
    av[1] = NULL;
    *av = NULL;
    va_start (ap, prog);
    while (noNULL (arg = va_arg (ap, char *))) {
	++ac;
	av = (char **) gresize(__FILE__, __LINE__, av, (ac + 1)*sizeof(char *));
	av[ac - 1] = arg;
    }
    av[ac] = NULL;
    switch ((child = fork ())) {
	case -1: /* ERROR */
	    vxlog (0, "**** fork() failed!\n");
	    return -1;
	case 0:  /* CHILD */
	    lfh = fileno (logfile);
	    unlessnull (logfile) {
		ofh = dup (1); efh = dup (2);
		dup2 (lfh, 1); dup2 (lfh, 2);
	    }
	    execvp (av[0], av);
	    unlessnull (logfile) {
		dup2 (ofh, 1); close (ofh);
		dup2 (efh, 2); close (efh);
	    }
	    vxlog (0, "**** execvp() failed: %s - %s\n",
		      av[0], strerror (errno));
	    exit (2);
	default: /* PARENT */
	    waitpid (child, &status, 0);
	    if (WIFSIGNALED (status)) {
		vxlog (0, "**** %s got signal #%d\n", av[0],
			  WTERMSIG (status));
		return 3;
	    }
	    if (WIFEXITED (status)) {
		if (WEXITSTATUS (status)) {
		    vxlog (0, "**** %s exited abnormally with code #%d\n",
			      WEXITSTATUS (status));
		}
		return WEXITSTATUS (status);
	    }
    }
    return 0;
}

mode_t reset_umask (void)
{
    return umask (0);
}

char *which (char *prog)
{
    char *PATH = getenv ("PATH"), *p, *q, *pp = NULL;
    struct stat s;
    ifnull (PATH) { return NULL; }
    PATH = gdupl (__FILE__, __LINE__, "PATH");
    p = PATH;
    while (noNULL(q = strchr (p, ':'))) {
	*q++ = '\0';
	pp = gconc (__FILE__, __LINE__, p, "/", prog, NULL);
	if (stat (pp, &s)) { gfree (pp); continue; }
	if (S_ISREG(s.st_mode) && (s.st_mode&(S_IXUSR|S_IXGRP|S_IXOTH))) {
	    gfree (PATH); return pp;
	}
	gfree (pp); p = q;
    }
    if (*p) {
	pp = gconc (__FILE__, __LINE__, p, "/", prog, NULL);
	if (!stat (pp, &s)) {
	    if (S_ISREG(s.st_mode) && (s.st_mode&(S_IXUSR|S_IXGRP|S_IXOTH))) {
		gfree (PATH); return pp;
	    }
	}
    }
    gfree (PATH); unlessnull (pp) { gfree (pp); }
    return NULL;
}

char *iftype (void)
{
    static size_t ifnsz = 41;
    static char *ifname = NULL;
    char *p = prefs.dbif_module;
    int ll;
    ifnull (ifname) { ifname = galloc (__FILE__, __LINE__, ifnsz); }
    *ifname = '\0';
    ifnull (p) { return ifname; }
    p = strrchr (p, '/');
    ifnull (p) { p = prefs.dbif_module; } else { ++p; }
    if (strlen (p) >= ifnsz) {
	ifnsz = strlen (p) + 1;
	ifname = gresize (__FILE__, __LINE__, p, ifnsz);
    }
    strcpy (ifname, p);
    ll = strlen (ifname);
    if (ll > 8 && (p = &ifname[ll - 8], !strcmp (p, "_dbif.so"))) {
	*p = '\0';
    } else if (ll > 3 && (p = &ifname[ll - 3], !strcmp (p, ".so"))) {
	*p = '\0';
    }
    return ifname;
}

FILE *ug_fopen (const char *path, const char *mode,
		const uid_t uid, const gid_t gid)
{
    FILE *res;
    unlessnull (res = fopen(path, mode)) { fchown(fileno(res), uid, gid); }
    return res;
}

void memdump (int ll, char *buf, int len)
{
    int ix, eol = 0;
    for (ix = 0; ix < len; ++ix) {
	eol = 0;
	vxlog (ll, " %.2X", ((int) buf[ix] & 0xff));
	if ((ix + 1) % 8 == 0) { vxlog (ll, "\n"); eol = 1; }
    }
    if (!eol) { vxlog (ll, "\n"); }
}
